PrevPeekable

Struct PrevPeekable 

pub struct PrevPeekable<I>
where I: Iterator, <I as Iterator>::Item: Clone,
{ /* private fields */ }
An iterator with prev(), prev_peek(), and peek() functions that return the previous element, a reference to the previous element, or a reference to the next element, respectively.

This struct is created by passing an Iterator whose Item implements Clone to the new function.

Implementations

Source

impl<I> PrevPeekable<I>
where I: Iterator, <I as Iterator>::Item: Clone,

pub fn new(iterator: I) -> Self

Creates a new PrevPeekable. It takes an Iterator whose Item implements Clone.

Examples

Basic usage:

use prev_iter::PrevPeekable;

let v = vec![1, 2, 3];
let mut iter = PrevPeekable::new(v.iter());

assert_eq!(Some(&1), iter.next());
assert_eq!(Some(&2), iter.next());
assert_eq!(Some(&1), iter.prev());
pub fn peek(&mut self) -> Option<&I::Item>

Returns a reference to the next() value without advancing the iterator.

Like next, if there is a value, it is wrapped in a Some(T). But if the iteration is over, None is returned.

Because peek() returns a reference, and many iterators iterate over references, there can be a possibly confusing situation where the return value is a double reference. You can see this effect in the examples below.

Examples

Basic usage:

let xs = [1, 2, 3];

let mut iter = xs.iter().peekable();

// peek() lets us see into the future
assert_eq!(iter.peek(), Some(&&1));
assert_eq!(iter.next(), Some(&1));

assert_eq!(iter.next(), Some(&2));

// The iterator does not advance even if we `peek` multiple times
assert_eq!(iter.peek(), Some(&&3));
assert_eq!(iter.peek(), Some(&&3));

assert_eq!(iter.next(), Some(&3));

// After the iterator is finished, so is `peek()`
assert_eq!(iter.peek(), None);
assert_eq!(iter.next(), None);
pub fn prev(&self) -> Option<I::Item>

Returns the previous value in the iterator without moving the iterator backwards. When the end is reached, it will always return the last element.

This function performs a clone() when returning the data.

Examples

Basic usage:

use prev_iter::PrevPeekable;

let v = vec![1, 2];
let mut it = PrevPeekable::new(v.iter());

// When the iterator is initialized there is not previous value
assert_eq!(None, it.prev());
assert_eq!(Some(&1), it.next());

// There is no value before the first element
assert_eq!(None, it.prev());
assert_eq!(Some(&2), it.next());

// Previous value before 2 is 1
assert_eq!(Some(&1), it.prev());

// The iterator doesn't have anymore values so the prev() will always
// return the last element
assert_eq!(None, it.next());
assert_eq!(Some(&2), it.prev());
pub fn prev_peek(&self) -> Option<&I::Item>

Returns a reference to the previous value in the iterator without moving the iterator backwards. When the end is reached, it will always return the last element.

Because prev_peek() returns a reference, and many iterators iterate over references, there can be a possibly confusing situation where the return value is a double reference. You can see this effect in the examples below.

Examples

Basic usage:

use prev_iter::PrevPeekable;

let v = vec![1, 2];
let mut it = PrevPeekable::new(v.iter());

// Initially there is nothing to peek at
assert_eq!(None, it.prev_peek());
assert_eq!(Some(&1), it.next());

// There is nothing before the first element
assert_eq!(None, it.prev_peek());
assert_eq!(Some(&2), it.next());

// 1 comes before 2
assert_eq!(Some(&&1), it.prev_peek());
assert_eq!(None, it.next());

// 2 will always be returned as the last element
assert_eq!(Some(&&2), it.prev_peek());